#include "main.h"

// Defines
#define shortPress 		 	100
#define longPress			350
#define fadingDelay		  	 10
#define idlePowerDown		 10
#define disablePowerDown	  0
#define noChangePowerDown  5400



// Global Variables

// ************* States **************
//    0x01 = second elapsed        0x10 =
//    0x02 =					   0x20 =
//    0x04 =					   0x40 =
//    0x08 =					   0x80 = Work LED
volatile uint8_t state;


// ************* Taster States **************
//   0x0001 = S1 short		0x0100 = S5 short
//	 0x0002 = S1 long		0x0200 = S5 long
//	 0x0004 = S2 short
//   0x0008 = S2 long
//   0x0010 = S3 short
//   0x0020 = S3 long
//   0x0040 = S4 short
//   0x0080 = S4 long
volatile uint16_t taster;

//  **************** Fading Directions & States ****************
//	  0x0001 = Fading Dir. Q1		0x0100 = Fading Q1 Done
//    0x0002 = Fading Dir. Q2		0x0200 = Fading Q2 Done
//    0x0004 = Fading Dir. Q3		0x0400 = Fading Q3 Done
//	  0x0008 = Fading Dir. Q4		0x0800 = Fading Q4 Done
//	  0x0010 = Fading Dir. Q5		0x1000 = Fading Q5 Done
volatile uint16_t fading;

// ************* Outputs States **************
//   0x01 = Q1 		0x10 = Q5
//	 0x02 = Q2
//	 0x04 = Q3
//   0x08 = Q4

volatile uint8_t outputs;


volatile uint16_t idleCount;
volatile uint16_t noChangeCount;

int main(void)
{

	uint8_t pwmc, pwm_q1, pwm_q2, pwm_q3, pwm_q4, pwm_q5;

	// Initialises the system clock
	SystemInit();

	// Systick Config running MCU with 48 MHz generates 1ms Tick
	SysTick_Config(48000);

	// Setting Defaults
	outputs = 0x0;
	fading = 0x0;
	pwmc = 0x0;
	pwm_q1 = 0x50;
	pwm_q2 = 0x50;
	pwm_q3 = 0x50;
	pwm_q4 = 0x50;
	pwm_q5 = 0x50;

	initInputs();
	initOutputs();
	initEeprom();
	initEXIT();

	pwm_q1 = ReadPWMSettings(q1);
	pwm_q2 = ReadPWMSettings(q2);
	pwm_q3 = ReadPWMSettings(q3);
	pwm_q4 = ReadPWMSettings(q4);
	pwm_q5 = ReadPWMSettings(q5);

    while(1)
    {
    	// Check if there is something to switch
    	if ((taster & 0x0001) == 0x0001)
    	{
    		if ((outputs & 0x01) == 0x01)
    		{
    			outputs = outputs & 0xFE;
    			GPIO_ResetBits(GPIOA,GPIO_Pin_3);
    		}
    		else
    		{
    			outputs = outputs | 0x01;
    		}
    		taster = taster & 0xFFFE;
    	}
    	if ((taster & 0x0004) == 0x0004)
    	{
    		if ((outputs & 0x02) == 0x02)
    		{
    			outputs = outputs & 0xFD;
    			GPIO_ResetBits(GPIOA,GPIO_Pin_4);
    		}
    		else
    		{
    			outputs = outputs | 0x02;
    		}
    		taster = taster & 0xFFFB;
    	}
    	if ((taster & 0x0010) == 0x0010)
    	{
    		if ((outputs & 0x04) == 0x04)
    		{
    			outputs = outputs & 0xFB;
    			GPIO_ResetBits(GPIOA,GPIO_Pin_5);
    		}
    		else
    		{
    			outputs = outputs | 0x04;
    		}
    		taster = taster & 0xFFEF;
    	}
    	if ((taster & 0x0040) == 0x0040)
    	{
    		if ((outputs & 0x08) == 0x08)
    		{
    			outputs = outputs & 0xF7;
    			GPIO_ResetBits(GPIOA,GPIO_Pin_6);
    		}
    		else
    		{
    			outputs = outputs | 0x08;
    		}
    		taster = taster & 0xFFBF;
    	}
    	if ((taster & 0x0100) == 0x0100)
    	{
    		if ((outputs & 0x10) == 0x10)
    		{
    			outputs = outputs & 0xEF;
    			GPIO_ResetBits(GPIOA,GPIO_Pin_7);
    		}
    		else
    		{
    			outputs = outputs | 0x10;
    		}
    		taster = taster & 0xFEFF;
    	}

    	// Check if i have to fade something
    	if (((taster & 0x0002) == 0x0002) & ((fading & 0x0100) != 0x0100))
    	{
    		if ((fading & 0x0001) != 0x0001)
    		{
    			if (pwm_q1 < 255)
    			{
    				pwm_q1++;
    			}
    			else
    			{
    				fading = fading | 0x0001;
    				pwm_q1--;
    			}
    		}
    		else
    		{
    			if (pwm_q1 > 0)
    			{
    				pwm_q1--;
    			}
    			else
    			{
    				fading = fading & 0xFFFE;
    				pwm_q1++;
    			}
    		}
    		fading = fading | 0x0100;
    		outputs |= 0x01;
    	}
    	if (((taster & 0x0008) == 0x0008) & ((fading & 0x0200) != 0x0200))
    	{
    		if ((fading & 0x0002) != 0x0002)
    		{
    			if (pwm_q2 < 255)
    			{
    				pwm_q2++;
    			}
    			else
    			{
    				fading = fading | 0x0002;
    				pwm_q2--;
    			}
    		}
    		else
    		{
    			if (pwm_q2 > 0)
    			{
    				pwm_q2--;
    			}
    			else
    			{
    				fading = fading & 0xFFFD;
    				pwm_q2++;
    			}
    		}
    		fading = fading | 0x0200;
    		outputs |= 0x02;
    	}
    	if (((taster & 0x0020) == 0x0020) & ((fading & 0x0400) != 0x0400))
    	{
    		if ((fading & 0x0004) != 0x0004)
    		{
    			if (pwm_q3 < 255)
    			{
    				pwm_q3++;
    			}
    			else
    			{
    				fading = fading | 0x0004;
    				pwm_q3--;
    			}
    		}
    		else
    		{
    			if (pwm_q3 > 0)
    			{
    				pwm_q3--;
    			}
    			else
    			{
    				fading = fading & 0xFFFB;
    				pwm_q3++;
    			}
    		}
    		fading = fading | 0x0400;
    		outputs |= 0x04;
    	}
    	if (((taster & 0x0080) == 0x0080) & ((fading & 0x0800) != 0x0800))
    	{
    		if ((fading & 0x0008) != 0x0008)
    		{
    			if (pwm_q4 < 255)
    			{
    				pwm_q4++;
    			}
    			else
    			{
    				fading = fading | 0x0008;
    				pwm_q4--;
    			}
    		}
    		else
    		{
    			if (pwm_q4 > 0)
    			{
    				pwm_q4--;
    			}
    			else
    			{
    				fading = fading & 0xFFF7;
    				pwm_q4++;
    			}
    		}
    		fading = fading | 0x0800;
    		outputs |= 0x08;
    	}
    	if (((taster & 0x0200) == 0x0200) & ((fading & 0x1000) != 0x1000))
    	{
    		if ((fading & 0x0010) != 0x0010)
    		{
    			if (pwm_q5 < 255)
    			{
    				pwm_q5++;
    			}
    			else
    			{
    				fading = fading | 0x0010;
    				pwm_q5--;
    			}
    		}
    		else
    		{
    			if (pwm_q5 > 0)
    			{
    				pwm_q5--;
    			}
    			else
    			{
    				fading = fading & 0xFFEF;
    				pwm_q5++;
    			}
    		}
    		fading = fading | 0x1000;
    		outputs |= 0x10;
    	}

    	// PWM Part
    	pwmc++;

   		if (pwmc > pwm_q1) GPIO_ResetBits(GPIOA,GPIO_Pin_3);
   		if (pwmc > pwm_q2) GPIO_ResetBits(GPIOA,GPIO_Pin_4);
   		if (pwmc > pwm_q3) GPIO_ResetBits(GPIOA,GPIO_Pin_5);
   		if (pwmc > pwm_q4) GPIO_ResetBits(GPIOA,GPIO_Pin_6);
   		if (pwmc > pwm_q5) GPIO_ResetBits(GPIOA,GPIO_Pin_7);

    	if (pwmc == 0)
    	{
    		if ((outputs & 0x01) == 0x01) GPIO_SetBits(GPIOA,GPIO_Pin_3);
    		if ((outputs & 0x02) == 0x02) GPIO_SetBits(GPIOA,GPIO_Pin_4);
    		if ((outputs & 0x04) == 0x04) GPIO_SetBits(GPIOA,GPIO_Pin_5);
    		if ((outputs & 0x08) == 0x08) GPIO_SetBits(GPIOA,GPIO_Pin_6);
    		if ((outputs & 0x10) == 0x10) GPIO_SetBits(GPIOA,GPIO_Pin_7);
    	}



    	if ((state & 0x01) == 0x01)
    	{
    		// Reset Marker
    		state = state & 0xFE;

    		// Check System Idle
    		if (outputs == 0x0)
    		{
    			idleCount++;
    		}
    		else
    		{
    			idleCount = 0;
    		}

    		// Check no Change Timer
    		noChangeCount++;

    		// Toggle Working LED
    		if ((state & 0x80) == 0x80)
    		{
    			state = state & 0x7F;
    			GPIO_SetBits(GPIOB, GPIO_Pin_1);
    		}
    		else
    		{
    			state = state | 0x80;
    			GPIO_ResetBits(GPIOB, GPIO_Pin_1);
    		}

    	}


    	// Check if System has to be powered down
    	if ((disablePowerDown != 1) & ((idleCount >= idlePowerDown) || (noChangeCount >= noChangePowerDown)))
    	{
    		WritePWMSettings(q1,pwm_q1);
    		WritePWMSettings(q2,pwm_q2);
    		WritePWMSettings(q3,pwm_q3);
    		WritePWMSettings(q4,pwm_q4);
    		WritePWMSettings(q5,pwm_q5);
    		PWR_EnterSTOPMode(PWR_Regulator_ON, PWR_STOPEntry_WFI);
    	}
    }
}


void SysTick_Handler(void)
{
	static uint16_t s1_count = 0;
	static uint16_t s2_count = 0;
	static uint16_t s3_count = 0;
	static uint16_t s4_count = 0;
	static uint16_t s5_count = 0;
	static uint8_t q1_fade = 0;
	static uint8_t q2_fade = 0;
	static uint8_t q3_fade = 0;
	static uint8_t q4_fade = 0;
	static uint8_t q5_fade = 0;

	static uint16_t second_count = 0;

	second_count++;

	if (second_count >= 1000)
	{
		second_count = 0;
		state = state | 0x01;
	}

	// Read S1
	if ((GPIO_ReadInputDataBit(GPIOF,GPIO_Pin_0) == 0x1) & ((taster & 0x0002) != 0x0002))
	{
		s1_count++;
		if (s1_count > longPress)
		{
			taster = taster | 0x0002;
			fading = fading ^ 0x0001;
			noChangeCount = 0;
		}
	}
	else if (GPIO_ReadInputDataBit(GPIOF,GPIO_Pin_0) == 0x0)
	{
		if ((s1_count > shortPress) & ((taster & 0x0002) != 0x0002))
		{
				noChangeCount = 0;
				taster = taster ^ 0x0001;
		}
		s1_count = 0;
		taster = taster & 0xFFFD;
	}

	// Read S2
	if ((GPIO_ReadInputDataBit(GPIOF,GPIO_Pin_1) == 0x1) & ((taster & 0x0008) != 0x0008))
	{
		s2_count++;
		if (s2_count > longPress)
		{
			taster = taster | 0x0008;
			fading = fading ^ 0x0002;
			noChangeCount = 0;
		}
	}
	else if (GPIO_ReadInputDataBit(GPIOF,GPIO_Pin_1) == 0x0)
	{
		if ((s2_count > shortPress) & ((taster & 0x0008) != 0x0008))
		{
			taster = taster ^ 0x0004;
			noChangeCount = 0;
		}
		s2_count = 0;
		taster = taster & 0xFFF7;
	}

	// Read S3
	if ((GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_0) == 0x1) & ((taster & 0x0020) != 0x0020))
	{
		s3_count++;
		if (s3_count > longPress)
		{
			taster = taster | 0x0020;
			fading = fading ^ 0x0004;
			noChangeCount = 0;
		}
	}
	else if (GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_0) == 0x0)
	{
		if ((s3_count > shortPress) & ((taster & 0x0020) != 0x0020))
		{
			taster = taster ^ 0x0010;
			noChangeCount = 0;
		}
		s3_count = 0;
		taster = taster & 0xFFDF;
	}

	// Read S4
	if ((GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_1) == 0x1) & ((taster & 0x0080) != 0x0080))
	{
		s4_count++;
		if (s4_count > longPress)
		{
			taster = taster | 0x0080;
			fading = fading ^ 0x0008;
			noChangeCount = 0;
		}
	}
	else if (GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_1) == 0x0)
	{
		if ((s4_count > shortPress) & ((taster & 0x0080) != 0x0080))
		{
			taster = taster ^ 0x0040;
			noChangeCount = 0;
		}
		s4_count = 0;
		taster = taster & 0xFF7F;
	}

	// Read S5
	if ((GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_2) == 0x1) & ((taster & 0x0200) != 0x0200))
	{
		s5_count++;
		if (s5_count > longPress)
		{
			taster = taster | 0x0200;
			fading = fading ^ 0x0010;
			noChangeCount = 0;
		}
	}
	else if (GPIO_ReadInputDataBit(GPIOA,GPIO_Pin_2) == 0x0)
	{
		if ((s5_count > shortPress) & ((taster & 0x0200) != 0x0200))
		{
			taster = taster ^ 0x0100;
			noChangeCount = 0;
		}
		s5_count = 0;
		taster = taster & 0xFDFF;
	}


	// Fading Delay
	if ((fading & 0x100) == 0x100)
	{
		q1_fade++;
		if (q1_fade >= fadingDelay)
		{
			q1_fade = 0;
			fading = fading & 0xFEFF;
		}
	}
	if ((fading & 0x200) == 0x200)
	{
		q2_fade++;
		if (q2_fade >= fadingDelay)
		{
			q2_fade = 0;
			fading = fading & 0xFDFF;
		}
	}
	if ((fading & 0x400) == 0x400)
	{
		q3_fade++;
		if (q3_fade >= fadingDelay)
		{
			q3_fade = 0;
			fading = fading & 0xFBFF;
		}
	}
	if ((fading & 0x800) == 0x800)
	{
		q4_fade++;
		if (q4_fade >= fadingDelay)
		{
			q4_fade = 0;
			fading = fading & 0xF7FF;
		}
	}
	if ((fading & 0x1000) == 0x1000)
	{
		q5_fade++;
		if (q5_fade >= fadingDelay)
		{
			q5_fade = 0;
			fading = fading & 0xEFFF;
		}
	}
}


void initInputs(void)
{
	GPIO_InitTypeDef io;

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOF, ENABLE);

	io.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2;
	io.GPIO_OType = GPIO_OType_PP;
	io.GPIO_PuPd = GPIO_PuPd_DOWN;
	io.GPIO_Mode = GPIO_Mode_IN;
	io.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA,&io);

	io.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	io.GPIO_OType = GPIO_OType_PP;
	io.GPIO_PuPd = GPIO_PuPd_DOWN;
	io.GPIO_Mode = GPIO_Mode_IN;
	io.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOF,&io);
}

void initOutputs(void)
{
	GPIO_InitTypeDef io;

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB, ENABLE);

	io.GPIO_Pin = GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	io.GPIO_OType = GPIO_OType_PP;
	io.GPIO_PuPd = GPIO_PuPd_NOPULL;
	io.GPIO_Mode = GPIO_Mode_OUT;
	io.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA,&io);

	io.GPIO_Pin = GPIO_Pin_1;
	io.GPIO_OType = GPIO_OType_PP;
	io.GPIO_PuPd = GPIO_PuPd_NOPULL;
	io.GPIO_Mode = GPIO_Mode_OUT;
	io.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB,&io);
}

void initEeprom(void)
{
	GPIO_InitTypeDef io;
	I2C_InitTypeDef i2c;

	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

	io.GPIO_Pin = GPIO_Pin_9 | GPIO_Pin_10;
	io.GPIO_OType = GPIO_OType_OD;
	io.GPIO_PuPd = GPIO_PuPd_NOPULL;
	io.GPIO_Mode = GPIO_Mode_AF;
	io.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA,&io);

	GPIO_PinAFConfig(GPIOA, GPIO_PinSource9, GPIO_AF_4);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_4);

	RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
	//RCC_I2CCLKConfig(RCC_I2C1CLK_SYSCLK);
	RCC_I2CCLKConfig(RCC_I2C1CLK_HSI);


	I2C_DeInit(I2C1);
	I2C_Cmd(I2C1, DISABLE);

	i2c.I2C_Ack = I2C_Ack_Enable;
	i2c.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	i2c.I2C_AnalogFilter = I2C_AnalogFilter_Enable;
	i2c.I2C_DigitalFilter = 0;
	i2c.I2C_Timing = 0x00401E26; ///0x10D55F7C;
	i2c.I2C_Mode = I2C_Mode_I2C;
	I2C_Init(I2C1, &i2c);

	I2C_Cmd(I2C1, ENABLE);
}

void initEXIT(void)
{
	  EXTI_InitTypeDef   EXTI_InitStructure;
	  NVIC_InitTypeDef   NVIC_InitStructure;

	  /* Enable SYSCFG clock */
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

	  /* Connect EXTI0 Line to PA0 pin */
	  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource0);
	  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource1);
	  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource2);

	  /* Configure EXTI0 line */
	  EXTI_InitStructure.EXTI_Line = EXTI_Line0 | EXTI_Line1 | EXTI_Line2;
	  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	  EXTI_Init(&EXTI_InitStructure);

	  /* Connect EXTI0 Line to PA0 pin */
	  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOF, EXTI_PinSource0);
	  SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOF, EXTI_PinSource1);

	  /* Configure EXTI0 line */
	  EXTI_InitStructure.EXTI_Line = EXTI_Line0 | EXTI_Line1;
	  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	  EXTI_Init(&EXTI_InitStructure);

      /* Enable and set Button EXTI Interrupt to the lowest priority */
      NVIC_InitStructure.NVIC_IRQChannel = EXTI0_1_IRQn;
      NVIC_InitStructure.NVIC_IRQChannelPriority = 0x03;
      NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

      NVIC_Init(&NVIC_InitStructure);

      NVIC_InitStructure.NVIC_IRQChannel = EXTI2_3_IRQn;
      NVIC_InitStructure.NVIC_IRQChannelPriority = 0x03;
      NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;

      NVIC_Init(&NVIC_InitStructure);

}

void EXTI0_1_IRQHandler(void)
{
  if(EXTI_GetITStatus(EXTI_Line0) != RESET)
  {
    /* Clear the USER Button EXTI line pending bit */
    EXTI_ClearITPendingBit(EXTI_Line0);
  }
  if(EXTI_GetITStatus(EXTI_Line1) != RESET)
  {
    /* Clear the USER Button EXTI line pending bit */
    EXTI_ClearITPendingBit(EXTI_Line1);
  }
}

void EXTI2_3_IRQHandler(void)
{
	  if(EXTI_GetITStatus(EXTI_Line2) != RESET)
	  {
	    /* Clear the USER Button EXTI line pending bit */
	    EXTI_ClearITPendingBit(EXTI_Line2);
	  }
	  if(EXTI_GetITStatus(EXTI_Line3) != RESET)
	  {
	    /* Clear the USER Button EXTI line pending bit */
	    EXTI_ClearITPendingBit(EXTI_Line3);
	  }}


void EEPRom_Write(uint8_t address, uint8_t reg, uint8_t data)
{

	//Wait until I2C isn't busy
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY) == SET);

	//"Handle" a transfer - The STM32F0 series has a shocking
	//I2C interface... Regardless! Send the address of the HMC
	//sensor down the I2C Bus and generate a start saying we're
	//going to write one byte. I'll be completely honest,
	//the I2C peripheral doesn't make too much sense to me
	//and a lot of the code is from the Std peripheral library
	I2C_TransferHandling(I2C1, address, 2, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);

	//Ensure the transmit interrupted flag is set
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_TXIS) == RESET);

	//Send the address of the register we wish to write to
	I2C_SendData(I2C1, (uint8_t)reg);

	//Ensure the transmit interrupted flag is set
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_TXIS) == RESET);

	I2C_SendData(I2C1, (uint8_t)data);

}

uint8_t EEPRom_Read(uint8_t address, uint8_t reg, uint8_t *Data, uint8_t amount)
{

    uint8_t ReadByte, Cnt;

    ReadByte = 0x0;

	//As per, ensure the I2C peripheral isn't busy!
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY) == SET);

	//Again, start another transfer using the "transfer handling"
	//function, the end bit being set in software this time
	//round, generate a start condition and indicate you will
	//be writing data to the HMC device.
	I2C_TransferHandling(I2C1, address, 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);

	//Wait until the transmit interrupt status is set
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_TXIS) == RESET);

	//Send the address of the register you wish to read
	I2C_SendData(I2C1, (uint8_t)reg);

	//Wait until transfer is complete!
	while(I2C_GetFlagStatus(I2C1, I2C_FLAG_TC) == RESET);

	//As per, start another transfer, we want to read DCnt
	//amount of bytes. Generate a start condition and
	//indicate that we want to read.
	I2C_TransferHandling(I2C1, address, amount, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);

	//Read in DCnt pieces of data
	for(Cnt = 0; Cnt<amount; Cnt++){

		//Wait until the RX register is full of luscious data!
        while(I2C_GetFlagStatus(I2C1, I2C_FLAG_RXNE) == RESET);
        //If we're only reading one byte, place that data direct into the
        //SingleData variable. If we're reading more than 1 piece of data
        //store in the array "Data" (a pointer from main)
        if(amount > 1) Data[Cnt] = I2C_ReceiveData(I2C1);
	else ReadByte = I2C_ReceiveData(I2C1);
     }

    //Wait for the stop condition to be sent
    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_STOPF) == RESET);

    //Clear the stop flag for next transfers
    I2C_ClearFlag(I2C1, I2C_FLAG_STOPF);

    return ReadByte;
}

uint8_t ReadPWMSettings(EEPROM_layout mode)
{
	uint8_t result;

	switch (mode)
	{
		case q1:
			result = EEPRom_Read(0xA1,0x00,result,1);
			break;
		case q2:
			result = EEPRom_Read(0xA1,0x01,result,1);
			break;
		case q3:
			result = EEPRom_Read(0xA1,0x02,result,1);
			break;
		case q4:
			result = EEPRom_Read(0xA1,0x03,result,1);
			break;
		case q5:
			result = EEPRom_Read(0xA1,0x04,result,1);
			break;
	}

	if ((result == 0xFF) || (result == 0x0)) result = 0x50;

	return result;
}

void WritePWMSettings(EEPROM_layout mode, uint8_t value)
{
	switch (mode)
	{
		case q1:
			EEPRom_Write(0xA0,0x00,value);
			break;
		case q2:
			EEPRom_Write(0xA0,0x01,value);
			break;
		case q3:
			EEPRom_Write(0xA0,0x02,value);
			break;
		case q4:
			EEPRom_Write(0xA0,0x03,value);
			break;
		case q5:
			EEPRom_Write(0xA0,0x04,value);
			break;
	}
}
